# StringPool 

A következő szorgalmi feladatban az erőforrásszűkében gyakran használt ObjectPool mintát fogjuk megvalósítani. Bár a minta általánosságban használható, illetve több nyelv alatt is elérhető valamilyen speciális változata (látsd: ThreadPool, ConnectionPool), mi ezek megvalósítására nem törekszünk, csak String-ek kezelésére fogjuk használni. 

A minta alkalmazására olyan esetben lehet szükség, ha viszonylag gyorsan, sok objektum keletkezik és szűnik meg. Ezek az objektumok létrehozása és felszabadítása pedig lassú is. Az objektumok tekinthetőek bizonyos erőforrásoknak is, amik szűk keresztmetszettel érhetők csak el. 

Általában ezek a következő alap fügvényekkel rendelkeznek:

 - Resource& acquire(): Azon erőforrásokból, amik szabadok a csoportban visszaad egy referenciát és innentől az adott erőforrás foglaltnak minősül. 
 - void release(Resource&): A kliens (hívó fél) visszaadja az objektumot, így az szabaddá válik.
 - Konstruktor, amivel az erőforráscsoport (pool) méretét lehet megadni 

Ugyanakkor a hagyományos "pooling" mintát kiterjesztjük további funkciókkal a String-ek kezelésére szabva: 
 - A konstruktorral nemcsak a pool-ban lévő string objektumok számát megadni, hanem azok kezdő méretét (mekkora legyen a karaktertömb) is. 
 - Egy string objektum elkérésekor megadható, hogy minimum mekkorának kell lennie a karaktertömb kapacitásának (ez nem a string hossza). 
 - Ha nincs megfelelő kapacitású string tömb, akkor az acquire függvény egy új stringet állít elő. 

## RString

Az első feladat egy `RString` nevű osztály elkészítése. Az osztálynak a következő publikus metódusokkal és konstruktorokkal kell rendelkeznie: 

```c++
class RString {
  public:
    // Létrehoz a megadott kapacitással egy karaktertömböt
    // A tömb első helyére egy \0-t rak
    RString(size_t_ capacity);
    // Létrehoz egy karaktertömböt strlen(str) + 1 mérettel
    // majd odamásolja str tartalmát 
    RString(const char* str);
    // Létrehoz egy karaktertömböt capacity mérettel,
    // majd odamásolja az str string-et
    // Ha a string nem fér bele, eldobja a neptun-kódodat
    RString(const char* str, size_t capacity)
    // Megadja a string méretét (strlen)
    size_t size();
    // Megadja a karaktertömb kapacitását
    size_t capacity();
    // Visszaadja a string memóriaterületére mutató pointert
    operator char*() const;
};
```

A `RString` osztály fenti publikus metódusai tovább nem bővíthetőek. természetesen gondoskodni kell a megfelelő dinamikus memóriakazelésről. 

## StringPool interfész



## Tiltott elemek

Az `RString` implementálásakor semmilyen STL tároló nem használható. A megoldásnál elemi dinamikus memóriakezelést kell alkalmazni. Ugyanakkor a `StringPool`-nál már használható valamilyen STL vektor vagy lista. 